Option Explicit


' --------------------------------------------------------
' NOT an API type -- used to pass information to a form
'   or sub from the gprocGetVolInfo sub
' --------------------------------------------------------
Type VolInfo
   VolName           As String * 256
   SystemType        As String * 32
   MaxFileNameLen    As Long
   SerialNum         As String * 24
   CasePreserved     As Integer
   CaseSensitive     As Integer
   Unicode           As Integer
   PersistentACLS    As Integer
   SupportFileComp   As Integer
   IsVolComp         As Integer
End Type


' --------------------------------------------------------
' NOT an API type -- used to pass information to a form
'   or sub from the gprocGetDiskFreeSpace sub
' --------------------------------------------------------
Type DiskInfo
   SectPerCluster    As Long
   BytesPerSector    As Long
   FreeClusters      As Long
   Clusters          As Long
End Type


' --------------------------------------------------------
' NOT an API type -- used to pass information to a form
'   or sub from the gprocGetFileTimeStamp sub
' --------------------------------------------------------
Type FileInfo
   DOS_Time          As String
   DOS_Date          As String
   Access_Time       As String
   Access_Date       As String
   Create_Time       As String
   Create_Date       As String
   Write_Time        As String
   Write_Date        As String
End Type


' --------------------------------------------------------
' Setup the TYPES needed by API calls
' --------------------------------------------------------
Type FILETIME           ' Used by GetFileTime and WIN32_FIND_DATA
   LowDateTime          As Long
   HighDateTime         As Long
End Type

Type WIN32_FIND_DATA    ' Used by FindFirstFile and FindNextFile
   dwFileAttributes     As Long
   ftCreationTime       As FILETIME
   ftLastAccessTime     As FILETIME
   ftLastWriteTime      As FILETIME
   nFileSizeHigh        As Long
   nFileSizeLow         As Long
   dwReserved0          As Long
   dwReserved1          As Long
   cFileName            As String * 260  'MUST be set to 260
   cAlternate           As String * 14
End Type

Type SYSTEMTIME         ' Used by FileTimeToSystemTime call
   Year                 As Integer
   Month                As Integer
   DayOfWeek            As Integer
   Day                  As Integer
   Hour                 As Integer
   Minute               As Integer
   Second               As Integer
   Milliseconds         As Integer
End Type


' --------------------------------------------------------
' API declare and constants used to set tab stops within
'   a listbox
' --------------------------------------------------------
Declare Function SendMessage Lib "User" (ByVal hWnd As Integer, ByVal wMsg As Integer, ByVal wParam As Integer, lparam As Any) As Long
Global Const WM_USER = &H400
Global Const LB_SETTABSTOPS = WM_USER + 19
Global Const LB_SETHORIZONTALEXTENT = (WM_USER + 21)


' --------------------------------------------------------
' API contants for file attributes
' --------------------------------------------------------
Const FILE_ATTRIBUTE_READONLY = &H1
Const FILE_ATTRIBUTE_HIDDEN = &H2
Const FILE_ATTRIBUTE_SYSTEM = &H4
Const FILE_ATTRIBUTE_DIRECTORY = &H10
Const FILE_ATTRIBUTE_ARCHIVE = &H20
Const FILE_ATTRIBUTE_NORMAL = &H80
Const FILE_ATTRIBUTE_TEMPORARY = &H100
Const FILE_ATTRIBUTE_COMPRESSED = &H800


' --------------------------------------------------------
' Constants for NTFS file systems
' --------------------------------------------------------
Const FS_CASE_SENSITIVE = 1
Const FS_CASE_IS_PRESERVED = 2
Const FS_UNICODE_STORED_ON_DISK = 4
Const FS_PERSISTENT_ACLS = 8
Const FS_FILE_COMPRESSION = &H10&
Const FS_VOLUME_IS_COMPRESSED = &H8000&


' --------------------------------------------------------
' GetDriveType return values
' --------------------------------------------------------
Const DRIVE_UNDETERMINED = 0
Const DRIVE_NO_ROOT = 1
Const DRIVE_REMOVABLE = 2
Const DRIVE_FIXED = 3
Const DRIVE_REMOTE = 4
Const DRIVE_CDROM = 5
Const DRIVE_RAMDISK = 6


' --------------------------------------------------------
' Error Status Codes for 32-bit API calls
' --------------------------------------------------------
Const INVALID_HANDLE_VALUE = -1
Const ERROR_SUCCESS = 0
Const ERROR_FILE_NOT_FOUND = 2&
Const ERROR_PATH_NOT_FOUND = 3&
Const ERROR_TOO_MANY_OPEN_FILES = 4&
Const ERROR_ACCESS_DENIED = 5&
Const ERROR_INVALID_HANDLE = 6&
Const ERROR_ARENA_TRASHED = 7&
Const ERROR_NOT_ENOUGH_MEMORY = 8&
Const ERROR_INVALID_DATA = 13&
Const ERROR_NO_MORE_FILES = 18&
Const ERROR_NOT_READY = 21&
Const ERROR_SHARING_VIOLATION = 32&
Const ERROR_INVALID_PARAMETER = 87&
Const ERROR_CALL_NOT_IMPLEMENTED = 120&
Const ERROR_INSUFFICIENT_BUFFER = 122&
Const ERROR_INVALID_NAME = 123&
Const ERROR_LABEL_TOO_LONG = 154&
Const ERROR_ALREADY_EXISTS = 183&
Const ERROR_NOACCESS = 998&


' --------------------------------------------------------
' 32-bit declares
' --------------------------------------------------------
Declare Function Declare32 Lib "CALL32.DLL" (ByVal func$, ByVal Library$, ByVal Args$) As Long
Declare Function FindFirstFileA Lib "CALL32.DLL" Alias "Call32" (ByVal lpFileName$, lpFindFileData As WIN32_FIND_DATA, ByVal Id As Long) As Long
Declare Function FindNextFileA Lib "CALL32.DLL" Alias "Call32" (ByVal hFindFile&, lpFindFileData As WIN32_FIND_DATA, ByVal Id As Long) As Long
Declare Function FindClose Lib "CALL32.DLL" Alias "Call32" (ByVal hFindFile&, ByVal Id As Long) As Long
Declare Function GetVolumeInformationA Lib "call32.dll" Alias "Call32" (ByVal lpRootPathName$, ByVal lpVolumeNameBuffer$, ByVal nVolumeNameSize&, lpVolumeSerialNumber&, lpMaximumComponentLength&, lpFileSystemFlags&, ByVal lpFileSystemNameBuffer$, ByVal nFileSystemNameSize&, ByVal Id As Long) As Long
Declare Function SetVolumeLabelA Lib "call32.dll" Alias "Call32" (ByVal lpRootPathName$, ByVal lpVolumeName$, ByVal Id As Long) As Long
Declare Function GetFileTime Lib "CALL32.DLL" Alias "Call32" (ByVal hFile As Long, lpCreationTime As FILETIME, lpLastAccessTime As FILETIME, lpLastWriteTime As FILETIME, ByVal Id As Long) As Long
Declare Function CloseHandle Lib "call32.dll" Alias "Call32" (ByVal hObject&, ByVal Id As Long) As Long
Declare Function FileTimeToDosDateTime Lib "call32.dll" Alias "Call32" (lpFileTime As FILETIME, lpFatDate As Long, lpFatTime As Long, ByVal Id As Long) As Long
Declare Function FileTimeToLocalFileTime Lib "call32.dll" Alias "Call32" (lpFileTime As FILETIME, lpLocalFileTime As FILETIME, ByVal Id As Long) As Long
Declare Function FileTimeToSystemTime Lib "call32.dll" Alias "Call32" (lpFileTime As FILETIME, lpLocalFileTime As SYSTEMTIME, ByVal Id As Long) As Long
Declare Function GetDiskFreeSpace Lib "call32.dll" Alias "Call32" (ByVal lpRootPathName As String, lpSectorsPerCluster As Long, lpBytesPerSector As Long, lpNumberOfFreeClusters As Long, lpTotalNumberOfClusters As Long, ByVal Id As Long) As Long
Declare Function GetDriveType Lib "call32.dll" Alias "Call32" (ByVal nDrive As String, ByVal Id As Long) As Long
Declare Function GetLastError Lib "CALL32.DLL" Alias "Call32" (ByVal Id As Long) As Long


' --------------------------------------------------------
' ID's used by Call32
' --------------------------------------------------------
Dim mlngFindFirstID        As Long
Dim mlngFindNextID         As Long
Dim mlngLastErrID          As Long
Dim mlngGetVolumeInfoID    As Long
Dim mlngSetVolLabelID      As Long
Dim mlngFindCloseID        As Long
Dim mlngGetFileTimeID      As Long
Dim mlngCloseHwndID        As Long
Dim mlngFileTimeDosID      As Long
Dim mlngTimeToLocalID      As Long
Dim mlngTimeToSystemTimeID As Long
Dim mlngDiskFreeSpaceID    As Long
Dim mlngGetDriveTypeID     As Long


' --------------------------------------------------------
' Global flag variable
' --------------------------------------------------------
Global gintIsShortNames    As Integer


' ********************************************************
'NOTE:   Delete from here down in declarations section if
'        you are using VB 4.0/16.  Also delete the line
'        vbNullChar = Chr$(0) in Form_Load of FrmLongF.Frm
' ********************************************************

' Show parameters
Global Const vbModal = 1
Global Const vbModeless = 0

' Function Parameters, MsgBox
Global Const vbCritical = 16      ' Critical message
Global Const vbQuestion = 32      ' Warning query
Global Const vbExclamation = 48   ' Warning message
Global Const vbInformation = 64   ' Information message

' MousePointer parameters
Global Const vbDefault = 0
Global Const vbHourGlass = 11

' This is a built-in constant in VB 4.0
Global vbNullChar                 ' Can't set constant to chr$(0)

' --------------------------------------------------------
' See if UniCode is supported on this disk partition.
' Return TRUE if supported, FALSE otherwise
' --------------------------------------------------------
Function gintfuncIsUnicodeSupported (xstrRootPath As String) As Integer
   Dim pstrVolName      As String   ' Name of the volume
   Dim pstrSystemType   As String   ' Name of the system (FAT, NTFS, etc)
   Dim plngVolNameSize  As Long     ' Size of the volume name
   Dim plngSysTypeSize  As Long     ' Size of system type
   Dim plngVolSerialNum As Long     ' Volume serial number
   Dim plngMaxNameLen   As Long     ' Max length of file names
   Dim plngSysFlags     As Long     ' System flags for volume
   Dim plngRtn          As Long     ' Return from function

   
   ' -----------------------------------------------------
   ' Initialize the strings, then call GetVolumeInformation
   '   function
   ' -----------------------------------------------------
   pstrVolName = Space$(256)
   pstrSystemType = Space$(32)
   plngSysTypeSize = CLng(Len(pstrSystemType))
   plngVolNameSize = CLng(Len(pstrVolName))
   plngRtn = GetVolumeInformationA(xstrRootPath, pstrVolName, plngVolNameSize, plngVolSerialNum, plngMaxNameLen, plngSysFlags, pstrSystemType, plngSysTypeSize, mlngGetVolumeInfoID)

   
   ' -----------------------------------------------------
   ' Examine returned system flag and set the return value
   ' -----------------------------------------------------
   If plngSysFlags And FS_UNICODE_STORED_ON_DISK Then
      gintfuncIsUnicodeSupported = True
   Else
      gintfuncIsUnicodeSupported = False
   End If

End Function

' --------------------------------------------------------
' Passed the root path (ie, "X:\", does require trailing
'   backslash)
' Return the disk information via the type variable:
'   xtypDiskInfo
' On failure, all values in type variable set to -1
' --------------------------------------------------------
Sub gprocGetDiskFreeSpace (xstrRootPath As String, xtypDiskInfo As DiskInfo)
   Dim plngRtn             As Long  ' Return value from GetDiskFreeSpace
   Dim plngSectPerCluster  As Long  ' \
   Dim plngBytesPerSector  As Long  '  \_ Disk parameters (total clusters
   Dim plngFreeClusters    As Long  '  /  free clusters, etc)
   Dim plngClusters        As Long  ' /
   Dim plngRtnErr          As Long  ' Return value from GetLastError

   
   ' -----------------------------------------------------
   ' Call the GetDiskFreeSpace function
   ' -----------------------------------------------------
   plngRtn = GetDiskFreeSpace(xstrRootPath, plngSectPerCluster, plngBytesPerSector, plngFreeClusters, plngClusters, mlngDiskFreeSpaceID)
   
   
   ' -----------------------------------------------------
   ' Function succeeded, fill in type variable
   ' -----------------------------------------------------
   If plngRtn <> False Then
      xtypDiskInfo.SectPerCluster = plngSectPerCluster
      xtypDiskInfo.BytesPerSector = plngBytesPerSector
      xtypDiskInfo.FreeClusters = plngFreeClusters
      xtypDiskInfo.Clusters = plngClusters
   
   
   ' -----------------------------------------------------
   ' Function failed, set everything to -1
   ' -----------------------------------------------------
   Else
      plngRtnErr = GetLastError(mlngLastErrID)
      MsgBox gstrfuncRtnAPI_Err(plngRtnErr), vbInformation, App.Title
      
      xtypDiskInfo.SectPerCluster = -1
      xtypDiskInfo.BytesPerSector = -1
      xtypDiskInfo.FreeClusters = -1
      xtypDiskInfo.Clusters = -1
   End If

End Sub

' --------------------------------------------------------
' Get the information on the file passed via the xstrPath
'   and xstrFileName parameters
' Return the file information in the type variable:
'   xtypFileInfo
' --------------------------------------------------------
Sub gprocGetFileTimeStamp (xstrPath As String, xstrFileName As String, xtypFileInfo As FileInfo)
   Dim Win32Data           As WIN32_FIND_DATA
   Dim plngFileHwnd        As Long        ' Handle returned from function
   Dim plngRtnErr          As Long        ' Long return value
   Dim ptypCreateTime      As FILETIME    ' \
   Dim ptypAccessTime      As FILETIME    '  \
   Dim ptypWriteTime       As FILETIME    '   >- Various file times
   Dim ptypLocalTime       As FILETIME    '  /
   Dim ptypSystemTime      As SYSTEMTIME  ' /
   Dim pintIsUnicode       As Integer     ' Is it Unicode volume?

   
   ' -----------------------------------------------------
   ' Find the first file in the selected directory
   ' -----------------------------------------------------
   plngFileHwnd = FindFirstFileA(xstrPath & xstrFileName, Win32Data, mlngFindFirstID)

   
   ' -----------------------------------------------------
   ' Get the 3 timestamps from the file
   ' -----------------------------------------------------
   If plngFileHwnd = INVALID_HANDLE_VALUE Then
      plngRtnErr = GetLastError(mlngLastErrID)
      MsgBox gstrfuncRtnAPI_Err(plngRtnErr), vbInformation, "File Error"
      xtypFileInfo.Create_Date = "No valid date"
      xtypFileInfo.Create_Time = "No valid time"
      xtypFileInfo.Access_Date = "No valid date"
      xtypFileInfo.Access_Time = "No valid time"
      xtypFileInfo.Write_Date = "No valid date"
      xtypFileInfo.Write_Time = "No valid time"
      Exit Sub
   Else
      ' --------------------------------------------------
      ' It is important that you close the handle
      ' --------------------------------------------------
      plngRtnErr = FindClose(plngFileHwnd, mlngFindCloseID)
      
      ptypCreateTime = Win32Data.ftCreationTime
      ptypAccessTime = Win32Data.ftLastAccessTime
      ptypWriteTime = Win32Data.ftLastWriteTime
   End If

   
   ' -----------------------------------------------------
   ' Check to see if UniCode is supported on this disk --
   ' If not, then don't try to get Create or Access
   '   times/dates
   ' Yes, root path needs trailing backslash!
   ' -----------------------------------------------------
   pintIsUnicode = gintfuncIsUnicodeSupported(Mid$(xstrPath, 1, 3))
   
   
   ' -----------------------------------------------------
   ' Get the VB date/time for the file
   ' -----------------------------------------------------
   xtypFileInfo.DOS_Date = Format$(FileDateTime(xstrPath & xstrFileName), "mm/dd/yyyy")
   xtypFileInfo.DOS_Time = Format$(FileDateTime(xstrPath & xstrFileName), "hh:nn:ss")

   
   ' -----------------------------------------------------
   ' Get the date/time for the file via API
   ' -----------------------------------------------------
   plngRtnErr = GetFileTime(plngFileHwnd, ptypCreateTime, ptypAccessTime, ptypWriteTime, mlngGetFileTimeID)

   
   ' -----------------------------------------------------
   ' Get the CREATE date/time for the file via API only
   '   if it supports Unicode
   ' -----------------------------------------------------
   If pintIsUnicode = True Then
      If FileTimeToLocalFileTime(ptypCreateTime, ptypLocalTime, mlngTimeToLocalID) <> False Then
         If FileTimeToSystemTime(ptypLocalTime, ptypSystemTime, mlngTimeToSystemTimeID) <> False Then
            If ptypSystemTime.Year = 1601 Then
               xtypFileInfo.Create_Date = "No valid date"
               xtypFileInfo.Create_Time = "No valid time"
            Else
               xtypFileInfo.Create_Date = gstrfuncGetDosDate(ptypSystemTime)
               xtypFileInfo.Create_Time = gstrfuncGetDosTime(ptypSystemTime)
            End If
         Else
            plngRtnErr = GetLastError(mlngLastErrID)
            If plngRtnErr <> 0 Then
               MsgBox gstrfuncRtnAPI_Err(plngRtnErr), vbInformation, "File Error"
            End If
         End If
      End If
   Else
      xtypFileInfo.Create_Date = "Not supported"
      xtypFileInfo.Create_Time = "Not supported"
   End If
   

   ' -----------------------------------------------------
   ' Get the ACCESS date/time for the file via API only
   '   if it supports Unicode
   ' -----------------------------------------------------
   If pintIsUnicode = True Then
      If FileTimeToLocalFileTime(ptypAccessTime, ptypLocalTime, mlngTimeToLocalID) <> False Then
         If FileTimeToSystemTime(ptypLocalTime, ptypSystemTime, mlngTimeToSystemTimeID) <> False Then
            If ptypSystemTime.Year = 1601 Then
               xtypFileInfo.Access_Date = "No valid date"
               xtypFileInfo.Access_Time = "No valid time"
            Else
               xtypFileInfo.Access_Date = gstrfuncGetDosDate(ptypSystemTime)
               xtypFileInfo.Access_Time = gstrfuncGetDosTime(ptypSystemTime)
            End If
         Else
            plngRtnErr = GetLastError(mlngLastErrID)
            If plngRtnErr <> 0 Then
               MsgBox gstrfuncRtnAPI_Err(plngRtnErr), vbInformation, "File Error"
            End If
         End If
      End If
   Else
      xtypFileInfo.Access_Date = "Not supported"
      xtypFileInfo.Access_Time = "Not supported"
   End If
   
   
   ' -----------------------------------------------------
   ' Get the WRITE date/time for the file via API --
   '   Should be same as date/time via VB call
   ' -----------------------------------------------------
   If FileTimeToLocalFileTime(ptypWriteTime, ptypLocalTime, mlngTimeToLocalID) <> False Then
      If FileTimeToSystemTime(ptypLocalTime, ptypSystemTime, mlngTimeToSystemTimeID) <> False Then
         xtypFileInfo.Write_Date = gstrfuncGetDosDate(ptypSystemTime)
         xtypFileInfo.Write_Time = gstrfuncGetDosTime(ptypSystemTime)
      Else
         plngRtnErr = GetLastError(mlngLastErrID)
         If plngRtnErr <> 0 Then
            MsgBox gstrfuncRtnAPI_Err(plngRtnErr), vbInformation, "File Error"
         End If
      End If
   End If

End Sub

' --------------------------------------------------------
' Passed the path of a directory and a filespec (normally
'   '*.*')
' Passed an uninitialized string array
'
' Reads all files in the directory with that filespec and
'   determines if they have a short filename.
' Fills the passed string array with either the long names
'   only or the short + long names.
' xintNumFiles returns the number of filesnames in the
'   array
' xintMaxChars returns the length of the longest line in
'   the array
' --------------------------------------------------------
Sub gprocGetLongFilename (xstrDirPath As String, xstrFileSpec As String, xintMaxChars As Integer, xaFileNames() As String, xintNumFiles As Integer)
   Dim Win32Data           As WIN32_FIND_DATA
   Dim pstrLongFName       As String   ' The LONG filename
   Dim pstrShortFName      As String   ' The SHORT filename
   Dim plngNextFileHwnd    As Long     ' Handle returned from function
   Dim plngFirstFileHwnd   As Long     ' Handle returned from function
   Dim plngRtnErr          As Long     ' Long return value
   Dim pintRtn             As Integer  ' Integer return value
   Dim pstrCombinedName    As String   ' Combined short + long name
   Dim pstrTab             As String   ' The TAB character
   Dim pintIsDir           As Integer  ' Is it a directory?
   Dim pstrAlwaysName      As String   ' Always filled in, even if only short name
   Dim plngAttrib          As Long     ' File attribute

   
   ' -----------------------------------------------------
   ' Initialize the variables
   ' -----------------------------------------------------
   pstrTab = Chr$(9)
   xintNumFiles = 0
   Erase xaFileNames

   
   ' -----------------------------------------------------
   ' Find the first file in the selected directory
   ' -----------------------------------------------------
   plngFirstFileHwnd = FindFirstFileA(xstrFileSpec, Win32Data, mlngFindFirstID)

   
   ' -----------------------------------------------------
   ' Get the long and short names (cut at first NULL) for
   '   the FIRST file
   ' -----------------------------------------------------
   pstrLongFName = Win32Data.cFileName
   pstrLongFName = Mid$(pstrLongFName, 1, InStr(pstrLongFName, vbNullChar) - 1)
   pstrShortFName = Win32Data.cAlternate
   pstrShortFName = Mid$(pstrShortFName, 1, InStr(pstrShortFName, vbNullChar) - 1)
   If pstrShortFName <> "" Then
      pstrAlwaysName = pstrShortFName
      pstrCombinedName = pstrShortFName & pstrTab & pstrLongFName
   Else
      pstrAlwaysName = pstrLongFName
      pstrCombinedName = pstrLongFName
   End If

   
   ' -----------------------------------------------------
   ' See if it is a directory
   ' -----------------------------------------------------
   If Win32Data.dwFileAttributes And FILE_ATTRIBUTE_DIRECTORY Then
      pintIsDir = True
   Else
      pintIsDir = False
   End If

   If pintIsDir = True Then
      ' --------------------------------------------------
      ' Do nothing -- don't want to show directories
      ' --------------------------------------------------
   Else
      ' --------------------------------------------------
      ' Increment maximum characters if needed
      ' --------------------------------------------------
      If Len(pstrCombinedName) > xintMaxChars Then
         xintMaxChars = Len(pstrCombinedName)
      End If
      
      ' --------------------------------------------------
      ' Increment counter and add to the array
      ' --------------------------------------------------
      xintNumFiles = xintNumFiles + 1
      ReDim Preserve xaFileNames(1 To xintNumFiles)
      xaFileNames(xintNumFiles) = LCase$(pstrCombinedName)
   End If


   ' -----------------------------------------------------
   ' Now loop and find the rest of the files
   ' -----------------------------------------------------
   Do While plngFirstFileHwnd <> 0
      plngNextFileHwnd = FindNextFileA(plngFirstFileHwnd, Win32Data, mlngFindNextID)

      If plngNextFileHwnd <> 0 Then
         pstrLongFName = Win32Data.cFileName
         pstrLongFName = Mid$(pstrLongFName, 1, InStr(pstrLongFName, vbNullChar) - 1)
         pstrShortFName = Win32Data.cAlternate
         pstrShortFName = Mid$(pstrShortFName, 1, InStr(pstrShortFName, vbNullChar) - 1)
         If pstrShortFName <> "" Then
            pstrAlwaysName = pstrShortFName
            pstrCombinedName = pstrShortFName & pstrTab & pstrLongFName
         Else
            pstrAlwaysName = pstrLongFName
            pstrCombinedName = pstrLongFName
         End If

         ' -----------------------------------------------
         ' See if it is a directory
         ' -----------------------------------------------
         If Win32Data.dwFileAttributes And FILE_ATTRIBUTE_DIRECTORY Then
            pintIsDir = True
         Else
            pintIsDir = False
         End If
         
         If pintIsDir = True Then
            ' --------------------------------------------
            ' Do nothing -- don't want to show directories
            ' --------------------------------------------
         Else
            ' --------------------------------------------
            ' Increment maximum characters if needed
            ' --------------------------------------------
            If Len(pstrCombinedName) > xintMaxChars Then
               xintMaxChars = Len(pstrCombinedName)
            End If
            
            ' --------------------------------------------
            ' Increment counter and add to the array
            ' --------------------------------------------
            xintNumFiles = xintNumFiles + 1
            ReDim Preserve xaFileNames(1 To xintNumFiles)
            xaFileNames(xintNumFiles) = LCase$(pstrCombinedName)
         End If

      Else
         ' -----------------------------------------------
         ' Get the error
         ' -----------------------------------------------
         plngRtnErr = GetLastError(mlngLastErrID)

         If plngRtnErr <> 0 Then
            ' --------------------------------------------
            ' Probably still want to show the error if
            '   there is a problem
            ' --------------------------------------------
            If plngRtnErr <> ERROR_NO_MORE_FILES Then
               MsgBox gstrfuncRtnAPI_Err(plngRtnErr), vbInformation, App.Title
               Exit Sub
            Else
               Exit Do
            End If
         End If

      End If
   Loop

   
   ' -----------------------------------------------------
   ' It is important that you close the handle for
   '   FindFirstFile
   ' -----------------------------------------------------
   plngRtnErr = FindClose(plngFirstFileHwnd, mlngFindCloseID)
   
End Sub

' --------------------------------------------------------
' Passed a drive root path (ie, "X:\", trailing backslash
'   needed)
' Return the volume info for this drive via type variable:
'   xtypVolInfo
' --------------------------------------------------------
Sub gprocGetVolInfo (xstrRootPath As String, xtypVol_Info As VolInfo)
   Dim pstrVolName         As String   ' Name of the volume
   Dim pstrSystemType      As String   ' Name of the system (FAT, NTFS, etc)
   Dim plngVolNameSize     As Long     ' Size of the volume name
   Dim plngSysTypeSize     As Long     ' Size of system type
   Dim plngVolSerialNum    As Long     ' Volume serial number
   Dim plngMaxFilenameLen  As Long     ' Max length of file names
   Dim plngSysFlags        As Long     ' System flags for volume
   Dim plngRtn             As Long     ' Return from function
   Dim plngRtnErr          As Long     ' To get the LastError

   
   ' -----------------------------------------------------
   ' Set the strings, then call GetVolumeInformation
   '   function
   ' -----------------------------------------------------
   pstrVolName = Space$(256)
   pstrSystemType = Space$(32)
   plngSysTypeSize = CLng(Len(pstrSystemType))
   plngVolNameSize = CLng(Len(pstrVolName))
   plngRtn = GetVolumeInformationA(xstrRootPath, pstrVolName, plngVolNameSize, plngVolSerialNum, plngMaxFilenameLen, plngSysFlags, pstrSystemType, plngSysTypeSize, mlngGetVolumeInfoID)

   
   ' -----------------------------------------------------
   ' If got an error, exit sub
   ' Otherwise, populate the type variable
   ' -----------------------------------------------------
   If plngRtn = False Then
      plngRtnErr = GetLastError(mlngLastErrID)
      If plngRtnErr <> 0 Then
         MsgBox gstrfuncRtnAPI_Err(plngRtnErr), vbInformation, App.Title
      End If
      Exit Sub
   
   Else
      xtypVol_Info.VolName = Trim$(pstrVolName)
      xtypVol_Info.SystemType = Trim$(pstrSystemType)
      xtypVol_Info.MaxFileNameLen = plngMaxFilenameLen
      xtypVol_Info.SerialNum = Hex$(plngVolSerialNum)


      If plngSysFlags And FS_CASE_IS_PRESERVED Then
         xtypVol_Info.CasePreserved = True
      Else
         xtypVol_Info.CasePreserved = False
      End If

      If plngSysFlags And FS_CASE_SENSITIVE Then
         xtypVol_Info.CaseSensitive = True
      Else
         xtypVol_Info.CaseSensitive = False
      End If

      If plngSysFlags And FS_UNICODE_STORED_ON_DISK Then
         xtypVol_Info.Unicode = True
      Else
         xtypVol_Info.Unicode = False
      End If

      If plngSysFlags And FS_PERSISTENT_ACLS Then
         xtypVol_Info.PersistentACLS = True
      Else
         xtypVol_Info.PersistentACLS = False
      End If

      If plngSysFlags And FS_FILE_COMPRESSION Then
         xtypVol_Info.SupportFileComp = True
      Else
         xtypVol_Info.SupportFileComp = False
      End If

      If plngSysFlags And FS_VOLUME_IS_COMPRESSED Then
         xtypVol_Info.IsVolComp = True
      Else
         xtypVol_Info.IsVolComp = False
      End If

   End If

End Sub

' --------------------------------------------------------
' Call this function before using any others
' Initializes the Call32 for all the 32-bit calls
' Only call once/program
' --------------------------------------------------------
Sub gprocInitRegistryCalls ()
   mlngLastErrID = Declare32("GetLastError", "kernel32", vbNullChar)
   mlngFindFirstID = Declare32("FindFirstFileA", "kernel32", "pp")
   mlngFindNextID = Declare32("FindNextFileA", "kernel32", "ip")
   mlngFindCloseID = Declare32("FindClose", "kernel32", "i")
   mlngGetVolumeInfoID = Declare32("GetVolumeInformationA", "kernel32", "ppippppi")
   mlngSetVolLabelID = Declare32("SetVolumeLabelA", "kernel32", "pp")
   mlngGetFileTimeID = Declare32("GetFileTime", "kernel32", "ippp")
   mlngCloseHwndID = Declare32("CloseHandle", "kernel32", "i")
   mlngFileTimeDosID = Declare32("FileTimeToDosDateTime", "kernel32", "ppp")
   mlngTimeToLocalID = Declare32("FileTimeToLocalFileTime", "kernel32", "pp")
   mlngTimeToSystemTimeID = Declare32("FileTimeToSystemTime", "kernel32", "pp")
   mlngDiskFreeSpaceID = Declare32("GetDiskFreeSpace", "kernel32", "ppppp")
   mlngGetDriveTypeID = Declare32("GetDriveType", "kernel32", "p")
End Sub

' --------------------------------------------------------
' Passed the type variable, format and return the date
'   value as "mm/dd/yyyy"
' --------------------------------------------------------
Function gstrfuncGetDosDate (xtypDate As SYSTEMTIME) As String
   Dim pstrTmp    As String

   pstrTmp = Format$(xtypDate.Month, "00") & "/"
   pstrTmp = pstrTmp & Format$(xtypDate.Day, "00") & "/"
   pstrTmp = pstrTmp & Format$(xtypDate.Year, "0000")
   gstrfuncGetDosDate = pstrTmp

End Function

' --------------------------------------------------------
' Passed the type variable, format and return the time
'   value as "hh:nn:ss"
' --------------------------------------------------------
Function gstrfuncGetDosTime (xtypeTime As SYSTEMTIME) As String
   Dim pstrTmp    As String
   
   pstrTmp = Format$(xtypeTime.Hour, "00") & ":"
   pstrTmp = pstrTmp & Format$(xtypeTime.Minute, "00") & ":"
   pstrTmp = pstrTmp & Format$(xtypeTime.Second, "00")
   gstrfuncGetDosTime = pstrTmp

End Function

' --------------------------------------------------------
' Passed an error number from GetLastError function
' Return the text associated with that error
' --------------------------------------------------------
Function gstrfuncRtnAPI_Err (xlngError As Long) As String

   Select Case xlngError
      Case ERROR_NOACCESS
         gstrfuncRtnAPI_Err = "Invalid access to memory location."
      Case ERROR_SHARING_VIOLATION
         gstrfuncRtnAPI_Err = "The process cannot access the file because it is being used by another process."
      Case ERROR_CALL_NOT_IMPLEMENTED
         gstrfuncRtnAPI_Err = "This function is only valid in Win32 mode."
      Case ERROR_INVALID_DATA
         gstrfuncRtnAPI_Err = "The data is invalid."
      Case ERROR_NO_MORE_FILES
         gstrfuncRtnAPI_Err = "There are no more files."
      Case ERROR_ALREADY_EXISTS
         gstrfuncRtnAPI_Err = "Cannot create a file when that file already exists."
      Case ERROR_INSUFFICIENT_BUFFER
         gstrfuncRtnAPI_Err = "The data area passed to a system call is too small."
      Case ERROR_TOO_MANY_OPEN_FILES
         gstrfuncRtnAPI_Err = "The system cannot open the file."
      Case ERROR_ACCESS_DENIED
         gstrfuncRtnAPI_Err = "Access is denied."
      Case ERROR_INVALID_HANDLE
         gstrfuncRtnAPI_Err = "The handle is invalid."
      Case ERROR_ARENA_TRASHED
         gstrfuncRtnAPI_Err = "The storage control blocks were destroyed."
      Case ERROR_NOT_ENOUGH_MEMORY
         gstrfuncRtnAPI_Err = "Not enough storage is available to process this command."
      Case ERROR_FILE_NOT_FOUND
         gstrfuncRtnAPI_Err = "The system cannot find the file specified."
      Case ERROR_PATH_NOT_FOUND
         gstrfuncRtnAPI_Err = "The system cannot find the path specified."
      Case ERROR_INVALID_PARAMETER
         gstrfuncRtnAPI_Err = "The parameter is incorrect."
      Case ERROR_INVALID_NAME
         gstrfuncRtnAPI_Err = "The filename, directory name, or volume label syntax is incorrect."
      Case Else
         gstrfuncRtnAPI_Err = "Unknown error #" & Trim$(Str$(xlngError))
   End Select

End Function

' --------------------------------------------------------
' Passed the root path of a drive, return a string
'   indicating the type of drive
' --------------------------------------------------------
Function gstrfuncRtnDrvType (xstrRootPath As String) As String
   Dim plngRtn    As Long
   
    
   plngRtn = GetDriveType(xstrRootPath, mlngGetDriveTypeID)
   Select Case plngRtn
      Case DRIVE_UNDETERMINED
         gstrfuncRtnDrvType = "Undetermined Type"
      Case DRIVE_NO_ROOT
         gstrfuncRtnDrvType = "No Root Directory"
      Case DRIVE_REMOVABLE
         gstrfuncRtnDrvType = "Removeable Drive"
      Case DRIVE_FIXED
         gstrfuncRtnDrvType = "Fixed Drive"
      Case DRIVE_REMOTE
         gstrfuncRtnDrvType = "Network Drive"
      Case DRIVE_CDROM
         gstrfuncRtnDrvType = "CD-ROM Drive"
      Case DRIVE_RAMDISK
         gstrfuncRtnDrvType = "RAM Disk"
   End Select

End Function

